varying highp vec2 v_texCoord;

/**
 * Engine provided uniform with the screen texture
 */
uniform sampler2D u_texture;

/**
 * x -> screen width
 * y -> screen height
 * z,w -> touch/mouse position.
 */
uniform vec4 u_screen_and_touch;
/**
 * In this shader this entire uniform is unused.
 * x -> time in seconds
 * y,z,w -> custom parameters
 */
uniform vec4 u_time_and_custom_params;

/**
 * The filter (expressed as a matrix of weights for each channel)
 * to be applied to the screen color.
 * Defined as 4 vec4 to bypass issues when loading shaders in FF.
 */
uniform vec4 u_red_weights;
uniform vec4 u_green_weights;
uniform vec4 u_blue_weights;
uniform vec4 u_alpha_weights;

vec4 doTheEffect(vec4 originalColor) {
    mat4 filterMatrix = mat4(
        u_red_weights,
        u_green_weights,
        u_blue_weights,
        u_alpha_weights
    );
    return originalColor * filterMatrix;
}

vec4 colorWithInput(vec2 touchPos, vec2 screenSize, vec4 originalColor, vec4 effectColor) {
    // Normalization factor for non squared screens.
    float normalizeFactor = screenSize.y / screenSize.x;

    // Normalized frag coordinates with the normalization factor applied.
    vec2 normFragCoord = gl_FragCoord.xy / screenSize;
    normFragCoord.y *= normalizeFactor;

    // Normalized touch coordinates with the normalization factor applied.
    vec2 normTouch = touchPos / screenSize;
    normTouch.y *= normalizeFactor;

    // Halo effect around touch position to see original colors.
    float factor = smoothstep(0.04, 0.2, length(normFragCoord - normTouch));
    return mix(originalColor, effectColor, factor);
}

void main()
{
    vec4 color = texture2D(u_texture, v_texCoord);
    vec2 touchPos = vec2(u_screen_and_touch.zw);
    vec2 screenSize = vec2(u_screen_and_touch.xy);

    gl_FragColor = colorWithInput(touchPos, screenSize, color, doTheEffect(color));
}